import functools

from paperpilot_common.middleware.server.context import get_trace_id
from paperpilot_common.utils.log import get_logger


class StubCacheWrapper:
    def __init__(self, stub):
        from paperpilot_common.grpc.client.cache import StubCache

        self.stub = stub
        self.stub_name = stub.__class__.__name__
        self.cache = StubCache(self.stub_name)
        self.logger = get_logger(f"grpc.client.cache_wrapper.{self.stub_name}")

    async def _call_method_and_cache(self, method, method_name, args, kwargs, use_cache, update_cache):
        """
        将grpc的方法调用转换为缓存方法调用

        :param method: grpc方法
        :param method_name: grpc方法名
        :param args: grpc方法参数
        :param kwargs: grpc方法参数
        :param use_cache: 是否使用缓存
        :param update_cache: 是否更新缓存
        :return: grpc方法返回值
        """
        if use_cache:
            # If using cache, check if the result is cached
            result = await self.cache.get_result(method_name, args, kwargs)
            if result:  # If cached, return the cached result
                self.logger.debug(f"cache hit: {method_name}")
                return result

        # If not cached, call the method and cache the result
        result = await method(*args, **kwargs)
        if update_cache:
            await self.cache.set_result(method_name, args, kwargs, result)

        return result

    def __getattr__(self, name):
        """
        将grpc的方法调用转换为缓存方法调用

        :param name: 方法名
        :return: 带缓存的方法
        """
        # Intercept any async method calls and call them through the cache
        method = getattr(self.stub, name, None)
        if method:

            @functools.wraps(method)
            async def wrapper(*args, **kwargs):
                use_cache = kwargs.pop("use_cache", False)
                update_cache = kwargs.pop("update_cache", True)
                return await self._call_method_and_cache(method, name, args, kwargs, use_cache, update_cache)

            return wrapper
        else:
            return method

    def __repr__(self):
        return f"<StubCacheWrapper {super().__repr__()}>"

    def __str__(self):
        return super().__str__


class StubTraceWrapper:
    def __init__(self, stub):
        self.stub = stub
        self.stub_name = stub.__class__.__name__
        self.logger = get_logger(f"grpc.client.trace_wrapper.{self.stub_name}")

    def __getattr__(self, name):
        # Intercept any async method calls and call them through the cache
        method = getattr(self.stub, name, None)
        if method:

            @functools.wraps(method)
            def wrapper(*args, **kwargs):
                trace_id = get_trace_id()
                # 寻找metadata
                metadata = kwargs.pop("metadata", list())

                found = False
                for item in metadata:
                    if item[0] == "x-trace-id":
                        found = True
                        item[1] = trace_id
                        break
                if not found:
                    metadata.append(("x-trace-id", trace_id.hex))

                kwargs["metadata"] = metadata

                return method(*args, **kwargs)

            return wrapper
        else:
            return method

    def __repr__(self):
        return f"<StubTraceWrapper {super().__repr__()}>"

    def __str__(self):
        return super().__str__



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2

from paperpilot_common.protobuf.user import auth_pb2 as paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2
from paperpilot_common.protobuf.user import user_pb2 as paperpilot__common_dot_protobuf_dot_user_dot_user__pb2


class AuthPublicServiceStub(object):
    """认证公开接口（可匿名访问）"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Login = channel.unary_unary(
            "/auth.AuthPublicService/Login",
            request_serializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.LoginRequest.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.LoginResponse.FromString,
        )
        self.Refresh = channel.unary_unary(
            "/auth.AuthPublicService/Refresh",
            request_serializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.RefreshTokenRequest.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.RefreshTokenResponse.FromString,
        )
        self.Logout = channel.unary_unary(
            "/auth.AuthPublicService/Logout",
            request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )
        self.Register = channel.unary_unary(
            "/auth.AuthPublicService/Register",
            request_serializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.CreateUserRequest.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.LoginResponse.FromString,
        )
        self.SendSmsCode = channel.unary_unary(
            "/auth.AuthPublicService/SendSmsCode",
            request_serializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.SendSmsCodeRequest.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )
        self.CountPhone = channel.unary_unary(
            "/auth.AuthPublicService/CountPhone",
            request_serializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.CountPhoneRequest.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.CountResponse.FromString,
        )
        self.CountUsername = channel.unary_unary(
            "/auth.AuthPublicService/CountUsername",
            request_serializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.CountUsernameRequest.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.CountResponse.FromString,
        )


class AuthPublicServiceServicer(object):
    """认证公开接口（可匿名访问）"""

    def Login(self, request, context):
        """登录"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Refresh(self, request, context):
        """刷新token"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Logout(self, request, context):
        """登出"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Register(self, request, context):
        """注册"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def SendSmsCode(self, request, context):
        """发送短信验证码"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CountPhone(self, request, context):
        """获取手机号数量"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CountUsername(self, request, context):
        """获取用户名数量"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_AuthPublicServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Login": grpc.unary_unary_rpc_method_handler(
            servicer.Login,
            request_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.LoginRequest.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.LoginResponse.SerializeToString,
        ),
        "Refresh": grpc.unary_unary_rpc_method_handler(
            servicer.Refresh,
            request_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.RefreshTokenRequest.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.RefreshTokenResponse.SerializeToString,
        ),
        "Logout": grpc.unary_unary_rpc_method_handler(
            servicer.Logout,
            request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
        "Register": grpc.unary_unary_rpc_method_handler(
            servicer.Register,
            request_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.CreateUserRequest.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.LoginResponse.SerializeToString,
        ),
        "SendSmsCode": grpc.unary_unary_rpc_method_handler(
            servicer.SendSmsCode,
            request_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.SendSmsCodeRequest.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
        "CountPhone": grpc.unary_unary_rpc_method_handler(
            servicer.CountPhone,
            request_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.CountPhoneRequest.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.CountResponse.SerializeToString,
        ),
        "CountUsername": grpc.unary_unary_rpc_method_handler(
            servicer.CountUsername,
            request_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.CountUsernameRequest.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.CountResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler("auth.AuthPublicService", rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class AuthPublicService(object):
    """认证公开接口（可匿名访问）"""

    @staticmethod
    def Login(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/auth.AuthPublicService/Login",
            paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.LoginRequest.SerializeToString,
            paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.LoginResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Refresh(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/auth.AuthPublicService/Refresh",
            paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.RefreshTokenRequest.SerializeToString,
            paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.RefreshTokenResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Logout(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/auth.AuthPublicService/Logout",
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Register(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/auth.AuthPublicService/Register",
            paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.CreateUserRequest.SerializeToString,
            paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.LoginResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def SendSmsCode(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/auth.AuthPublicService/SendSmsCode",
            paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.SendSmsCodeRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def CountPhone(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/auth.AuthPublicService/CountPhone",
            paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.CountPhoneRequest.SerializeToString,
            paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.CountResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def CountUsername(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/auth.AuthPublicService/CountUsername",
            paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.CountUsernameRequest.SerializeToString,
            paperpilot__common_dot_protobuf_dot_user_dot_auth__pb2.CountResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2

from paperpilot_common.protobuf.paper import paper_pb2 as paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2


class PaperServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.AddPaper = channel.unary_unary(
            "/paper.PaperService/AddPaper",
            request_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.FromString,
        )
        self.UpdatePaper = channel.unary_unary(
            "/paper.PaperService/UpdatePaper",
            request_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.FromString,
        )
        self.UpdateAttachment = channel.unary_unary(
            "/paper.PaperService/UpdateAttachment",
            request_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.UpdateAttachmentRequest.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.FromString,
        )


class PaperServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def AddPaper(self, request, context):
        """添加论文"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdatePaper(self, request, context):
        """更新论文"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdateAttachment(self, request, context):
        """更新论文附件"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_PaperServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "AddPaper": grpc.unary_unary_rpc_method_handler(
            servicer.AddPaper,
            request_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.SerializeToString,
        ),
        "UpdatePaper": grpc.unary_unary_rpc_method_handler(
            servicer.UpdatePaper,
            request_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.SerializeToString,
        ),
        "UpdateAttachment": grpc.unary_unary_rpc_method_handler(
            servicer.UpdateAttachment,
            request_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.UpdateAttachmentRequest.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler("paper.PaperService", rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class PaperService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def AddPaper(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/paper.PaperService/AddPaper",
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.SerializeToString,
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UpdatePaper(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/paper.PaperService/UpdatePaper",
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.SerializeToString,
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UpdateAttachment(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/paper.PaperService/UpdateAttachment",
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.UpdateAttachmentRequest.SerializeToString,
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class PaperPublicServiceStub(object):
    """论文公开接口"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ListPaper = channel.unary_unary(
            "/paper.PaperPublicService/ListPaper",
            request_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.ListPaperRequest.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.ListPaperResponse.FromString,
        )
        self.GetPaper = channel.unary_unary(
            "/paper.PaperPublicService/GetPaper",
            request_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperId.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.FromString,
        )
        self.CreatePaper = channel.unary_unary(
            "/paper.PaperPublicService/CreatePaper",
            request_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.CreatePaperRequest.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.FromString,
        )
        self.CreatePaperByLink = channel.unary_unary(
            "/paper.PaperPublicService/CreatePaperByLink",
            request_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.CreatePaperByLinkRequest.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.FromString,
        )
        self.UpdatePaper = channel.unary_unary(
            "/paper.PaperPublicService/UpdatePaper",
            request_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.FromString,
        )
        self.UploadAttachment = channel.unary_unary(
            "/paper.PaperPublicService/UploadAttachment",
            request_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.UploadAttachmentRequest.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.UploadAttachmentResponse.FromString,
        )
        self.DeletePaper = channel.unary_unary(
            "/paper.PaperPublicService/DeletePaper",
            request_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperId.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )


class PaperPublicServiceServicer(object):
    """论文公开接口"""

    def ListPaper(self, request, context):
        """获取论文列表"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetPaper(self, request, context):
        """获取论文详情"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CreatePaper(self, request, context):
        """添加论文"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CreatePaperByLink(self, request, context):
        """从链接创建论文"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdatePaper(self, request, context):
        """更新论文"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UploadAttachment(self, request, context):
        """上传附件"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeletePaper(self, request, context):
        """删除论文"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_PaperPublicServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "ListPaper": grpc.unary_unary_rpc_method_handler(
            servicer.ListPaper,
            request_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.ListPaperRequest.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.ListPaperResponse.SerializeToString,
        ),
        "GetPaper": grpc.unary_unary_rpc_method_handler(
            servicer.GetPaper,
            request_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperId.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.SerializeToString,
        ),
        "CreatePaper": grpc.unary_unary_rpc_method_handler(
            servicer.CreatePaper,
            request_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.CreatePaperRequest.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.SerializeToString,
        ),
        "CreatePaperByLink": grpc.unary_unary_rpc_method_handler(
            servicer.CreatePaperByLink,
            request_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.CreatePaperByLinkRequest.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.SerializeToString,
        ),
        "UpdatePaper": grpc.unary_unary_rpc_method_handler(
            servicer.UpdatePaper,
            request_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.SerializeToString,
        ),
        "UploadAttachment": grpc.unary_unary_rpc_method_handler(
            servicer.UploadAttachment,
            request_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.UploadAttachmentRequest.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.UploadAttachmentResponse.SerializeToString,
        ),
        "DeletePaper": grpc.unary_unary_rpc_method_handler(
            servicer.DeletePaper,
            request_deserializer=paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperId.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler("paper.PaperPublicService", rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class PaperPublicService(object):
    """论文公开接口"""

    @staticmethod
    def ListPaper(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/paper.PaperPublicService/ListPaper",
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.ListPaperRequest.SerializeToString,
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.ListPaperResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetPaper(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/paper.PaperPublicService/GetPaper",
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperId.SerializeToString,
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def CreatePaper(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/paper.PaperPublicService/CreatePaper",
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.CreatePaperRequest.SerializeToString,
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def CreatePaperByLink(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/paper.PaperPublicService/CreatePaperByLink",
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.CreatePaperByLinkRequest.SerializeToString,
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UpdatePaper(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/paper.PaperPublicService/UpdatePaper",
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.SerializeToString,
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperDetail.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UploadAttachment(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/paper.PaperPublicService/UploadAttachment",
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.UploadAttachmentRequest.SerializeToString,
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.UploadAttachmentResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeletePaper(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/paper.PaperPublicService/DeletePaper",
            paperpilot__common_dot_protobuf_dot_paper_dot_paper__pb2.PaperId.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



from django.contrib.admin.models import LogEntry


def custom_render(self, context):
    entries = LogEntry.objects.all()
    context[self.varname] = entries.select_related("content_type")[: int(self.limit)]
    return ""



from asgiref.sync import sync_to_async
from django.db.transaction import Atomic


class AsyncAtomicContextManager(Atomic):
    def __init__(self, using=None, savepoint=True, durable=False):
        super().__init__(using, savepoint, durable)

    async def __aenter__(self):
        await sync_to_async(super().__enter__)()
        return self

    async def __aexit__(self, exc_type, exc_value, traceback):
        await sync_to_async(super().__exit__)(exc_type, exc_value, traceback)


def aatomic(fun, *args, **kwargs):
    async def wrapper():
        async with AsyncAtomicContextManager():
            await fun(*args, **kwargs)

    return wrapper



import base64
from typing import AnyStr
from urllib.parse import unquote

import aiohttp
from Crypto.Hash import MD5
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5

from paperpilot_common.utils.log import get_logger


class OssCallbackChecker:
    HTTP_ADDR = "http://gosspublic.alicdn.com/"
    HTTPS_ADDR = "https://gosspublic.alicdn.com/"

    logger = get_logger("oss.callback.checker")
    cache = None
    pub_key = {}
    verifier = {}

    async def _get_pub_key_online(self, pub_key_url: str) -> AnyStr:
        """
        从网络获取pub key
        """
        self.logger.debug(f"get pub key online from {pub_key_url}")
        async with aiohttp.ClientSession() as session:
            async with session.get(pub_key_url) as response:
                if response.status == 200:
                    file_data = await response.read()
                    return file_data
                else:
                    raise Exception(f"get pub key online failed, status: {response.status}")

    async def get_pub_key(self, pub_key_url: str) -> AnyStr:
        """
        获取公钥
        :param pub_key_url: url
        :return:
        """
        key = f"oss:pub_key:{pub_key_url}"

        try:
            res = await self.cache.get(key, None)
            if res is None:
                res = self._get_pub_key_online(pub_key_url)
                await self.cache.set(key, res)
            return res
        except Exception as e:
            self.logger.warning(f"get pub key failed, url: {pub_key_url}, fallback to local cache")
            self.logger.exception(e)

            if self.pub_key.get(pub_key_url, None) is None:
                self.pub_key[pub_key_url] = await self._get_pub_key_online(pub_key_url)

            return self.pub_key[pub_key_url]

    async def get_verifier(self, pub_key_url_base64: str) -> PKCS1_v1_5:
        """
        获取验证器
        :param pub_key_url_base64: url base64
        :return:
        """
        if self.verifier.get(pub_key_url_base64, None) is None:
            # 对x-oss-pub-key-url做base64解码后获取到公钥
            pub_key_url = base64.b64decode(pub_key_url_base64).decode()

            # 为了保证该public_key是由OSS颁发的，用户需要校验x-oss-pub-key-url的开头
            if not pub_key_url.startswith(self.HTTP_ADDR) and not pub_key_url.startswith(self.HTTPS_ADDR):
                raise Exception("pub_key_url is invalid")
            pub_key = await self.get_pub_key(pub_key_url)
            rsa_pub = RSA.importKey(pub_key)
            self.verifier[pub_key_url_base64] = PKCS1_v1_5.new(rsa_pub)

        return self.verifier[pub_key_url_base64]

    async def check_callback_signature(
        self, headers: dict[str, str], query_string: str, path_info: str, body: bytes
    ) -> bool:
        """
        检测回调身份

        :param headers: 请求头
        :param query_string: 查询字符串
        :param path_info: 路径
        :param body: 请求体
        :return: 是否通过
        """
        authorization_base64 = headers.get("AUTHORIZATION", None)  # 获取AUTHORIZATION
        pub_key_url_base64 = headers.get("X-OSS-PUB-KEY-URL", None)  # 获取公钥
        if authorization_base64 is None or pub_key_url_base64 is None:
            return False

        try:
            verifier = await self.get_verifier(pub_key_url_base64)

            # 获取base64解码后的签名
            authorization = base64.b64decode(authorization_base64)

            # 获取待签名字符串
            callback_body = body

            if query_string == "":
                auth_str = unquote(path_info) + "\n" + callback_body.decode()
            else:
                auth_str = unquote(path_info) + "?" + query_string + "\n" + callback_body.decode()

            # 验证签名
            auth_md5 = MD5.new(auth_str.encode())
            return verifier.verify(auth_md5, authorization)
        except Exception:
            return False



import sys
from abc import ABCMeta
from concurrent import futures
from pathlib import Path

import grpc
import grpc.aio
import grpc_health.v1.health as health
import grpc_reflection.v1alpha.reflection as reflection
from django.conf import settings
from django.utils.module_loading import import_string
from grpc_health.v1 import health_pb2, health_pb2_grpc
from grpc_reflection.v1alpha import reflection_pb2_grpc

from paperpilot_common.grpc.signals.wrapper import SignalWrapper
from paperpilot_common.middleware.server.db import DBMiddleware
from paperpilot_common.utils.log import get_logger

logger = get_logger("server.grpc.util")


def get_existed_file_path(path: str) -> Path | None:
    if path is None:
        return None

    path = Path(path)

    # 如果路径是绝对路径并且存在，则返回 Path 对象
    if path.is_absolute() and path.exists():
        return path

    # 获取sys.path中的路径列表
    for search_path in sys.path:
        current_path = Path(search_path) / path
        if current_path.exists():
            return current_path

        # 如果未找到文件或路径不存在，则返回 None
    return None


def create_server(address):
    config = getattr(settings, "GRPC_SERVER", dict())
    servicers_list = config.get("servicers", [])  # callbacks to add servicers to the server
    interceptors = load_interceptors(config.get("interceptors", []))
    max_workers = config.get("max_workers", 10)
    maximum_concurrent_rpcs = config.get("maximum_concurrent_rpcs", None)
    options = config.get("options", [])
    is_async = config.get("async", False)
    server_reflection = config.get("reflection", True)

    key_path = get_existed_file_path(config.get("ssl_key", None))
    cert_path = get_existed_file_path(config.get("ssl_cert", None))

    # create a gRPC server
    if is_async is True:
        server = grpc.aio.server(
            interceptors=interceptors, maximum_concurrent_rpcs=maximum_concurrent_rpcs, options=options
        )
    else:
        server = grpc.server(
            thread_pool=futures.ThreadPoolExecutor(max_workers=max_workers),
            interceptors=interceptors,
            maximum_concurrent_rpcs=maximum_concurrent_rpcs,
            options=options,
        )

    service_names = add_servicers(server, servicers_list)

    if server_reflection:
        service_names.append(reflection.SERVICE_NAME)
        reflection_servicer = reflection.aio.ReflectionServicer(service_names)
        reflection_pb2_grpc.add_ServerReflectionServicer_to_server(reflection_servicer, server)
        logger.info("gRPC server reflection enabled")

    if key_path is None or cert_path is None:
        server.add_insecure_port(address)
        logger.info("gRPC server listening on %s", address)
    else:
        key_path = get_existed_file_path(getattr(settings, "GRPC_SSL_KEY", None))
        cert_path = get_existed_file_path(getattr(settings, "GRPC_SSL_CERT", None))

        logger.info(f"Get ssl key path: {key_path}")
        logger.info(f"Get ssl cert path: {cert_path}")

        with open(key_path, "rb") as f:
            private_key = f.read()
        with open(cert_path, "rb") as f:
            certificate_chain = f.read()

        creds = grpc.ssl_server_credentials(
            [
                (
                    private_key,
                    certificate_chain,
                ),
            ]
        )

        server.add_secure_port(address, creds)
        logger.info("gRPC server with ssl listening on %s", address)

    return server


def add_servicers(server, servicers_list):
    """
    Add servicers to the server
    """
    ps = SignalWrapper(server)
    if len(servicers_list) == 0:
        logger.warning("No servicers configured. Did you add GRPSERVER['servicers'] list to settings?")

    services_names = []

    for path in servicers_list:
        logger.debug("Adding servicers from %s", path)
        callback = import_string(path)
        service_name = callback(ps)

        if not service_name:
            continue

        if isinstance(service_name, str):
            service_name = [service_name]

        services_names.extend(service_name)

    # add health check
    health_servicer = health.aio.HealthServicer()
    health_pb2_grpc.add_HealthServicer_to_server(health_servicer, server)
    services_names.append(health_pb2.DESCRIPTOR.services_by_name["Health"].full_name)

    return services_names


def load_interceptors(strings) -> list:
    # Default interceptors
    result = []
    # User defined interceptors
    for path in strings:
        logger.debug("Initializing interceptor from %s", path)
        result.append(import_string(path)())
    result.append(DBMiddleware())
    return result


def extract_handlers(server):
    for handler in server._state.generic_handlers:
        for path, it in handler._method_handlers.items():
            unary = it.unary_unary
            if unary is None:
                name = "???"
                params = "???"
                abstract = "DOES NOT EXIST"
            else:
                code = it.unary_unary.__code__
                name = code.co_name
                params = ", ".join(code.co_varnames)
                abstract = ""
                if isinstance(it.__class__, ABCMeta):
                    abstract = "NOT IMPLEMENTED"

            yield "{path}: {name}({params}) {abstract}".format(path=path, name=name, params=params, abstract=abstract)



import datetime
import random
import time
from typing import AnyStr, Tuple
from urllib.request import urlopen

from django.core.cache import cache
from django.db.models import TextChoices


class OssDirectToken:
    access_key_id: str
    host: str
    policy: str
    signature: str
    callback: str

    def __init__(self, access_key_id: str, host: str, policy: str, signature: str, callback: str):
        self.access_key_id = access_key_id
        self.host = host
        self.policy = policy
        self.signature = signature
        self.callback = callback

    def to_protobuf(self):
        from paperpilot_common.protobuf.common.util_pb2 import OssToken

        return OssToken(
            access_key_id=self.access_key_id,
            host=self.host,
            policy=self.policy,
            signature=self.signature,
            callback_body=self.callback,
        )


class ConditionCompare(TextChoices):
    EQ = "eq"
    START_WITH = "starts-with"
    IN = "in"
    NOT_IN = "not-in"


class Condition:
    value: str | list[str]
    compare: ConditionCompare

    def __init__(self, value: str | list[str], compare: ConditionCompare = None):
        if compare:  # 传入比较类型
            self.value = value
            self.compare = compare
        else:  # 未传入比较类型
            if isinstance(value, list):  # 如果是列表
                self.value = value
                self.compare = ConditionCompare.IN  # 默认为in
            else:  # 如果是字符串
                if value.endswith("*"):  # 如果是以*结尾
                    self.value = value[:-1]
                    self.compare = ConditionCompare.START_WITH  # 默认为starts-with
                else:  # 普通字符串
                    self.value = value
                    self.compare = ConditionCompare.EQ  # 默认为eq

    def parse(self, key: str) -> list[str | list[str]]:
        return [self.compare.value, f"${key}", self.value]


def escape_special_characters(input_str: str) -> str:
    escaped_str = input_str

    # 使用转义字符替换特殊字符
    escaped_str = escaped_str.replace("\\", "\\\\")  # 反斜杠
    escaped_str = escaped_str.replace("/", "\\/")  # 斜杠
    escaped_str = escaped_str.replace('"', '\\"')  # 双引号
    escaped_str = escaped_str.replace("$", "\\$")  # 美元符
    escaped_str = escaped_str.replace("\b", "\\b")  # 空格
    escaped_str = escaped_str.replace("\f", "\\f")  # 换页
    escaped_str = escaped_str.replace("\n", "\\n")  # 换行
    escaped_str = escaped_str.replace("\r", "\\r")  # 回车
    escaped_str = escaped_str.replace("\t", "\\t")  # 水平制表符

    return escaped_str


def handle_condition(conditions: list[list], key: str, value: str | list[str] | Condition | None) -> None:
    """
    处理条件
    """
    if not value:
        return

    if isinstance(value, str) or isinstance(value, list):
        condition = Condition(value)
    else:
        condition = value

    conditions.append(condition.parse(key))


def parse_size(size: str | int) -> int:
    """
    解析大小

    :param size: 大小
    :return: 字节数
    """
    if isinstance(size, int):
        return size

    size = size.strip().lower()
    multipliers = {"b": 1, "k": 1024, "m": 1024**2, "g": 1024**3, "t": 1024**4, "p": 1024**5}

    if size[-1] == "b":
        size = size[:-1]

    if size == "":
        return 0

    try:
        if size.isdigit():
            return int(size)
        for unit, multiplier in multipliers.items():
            if size.endswith(unit):
                num = float(size[:-1])
                return int(num * multiplier)
        raise ValueError("Invalid size format")
    except ValueError:
        raise ValueError("Invalid size format")


def get_iso_8601(expire: float) -> str:
    gmt = datetime.datetime.utcfromtimestamp(expire).isoformat()
    gmt += "Z"
    return gmt


def split_file_name(file_name: str) -> Tuple[str, str]:
    """
    获取文件名与扩展名

    :param file_name: 文件全名

    :return: 文件名，扩展名
    """
    if "." in file_name:  # 文件存在扩展名
        ext = file_name.split(".")[-1]  # 文件扩展名
        name = ".".join(file_name.split(".")[0:-1])
    else:
        ext = ""
        name = file_name

    return name, ext


def get_random_name(file_name: str) -> str:
    """
    获取随机文件名

    :param file_name: 原文件名

    :return:
    """
    name, ext = split_file_name(file_name)

    new_name = time.strftime("%Y%m%d%H%M%S")  # 定义文件名，年月日时分秒随机数
    new_name = new_name + "_%04d" % random.randint(0, 10000) + (("." + ext) if ext != "" else "")

    return new_name



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: paperpilot_common/protobuf/user/user.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2

from paperpilot_common.protobuf.common import util_pb2 as paperpilot__common_dot_protobuf_dot_common_dot_util__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n*paperpilot_common/protobuf/user/user.proto\x12\x04user\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a,paperpilot_common/protobuf/common/util.proto"\x14\n\x06UserId\x12\n\n\x02id\x18\x01 \x01(\t"8\n\x08UserInfo\x12\n\n\x02id\x18\x01 \x01(\t\x12\x10\n\x08username\x18\x02 \x01(\t\x12\x0e\n\x06\x61vatar\x18\x03 \x01(\t"\xab\x01\n\nUserDetail\x12\n\n\x02id\x18\x01 \x01(\t\x12\x10\n\x08username\x18\x02 \x01(\t\x12\r\n\x05phone\x18\x03 \x01(\t\x12\x0e\n\x06\x61vatar\x18\x04 \x01(\t\x12/\n\x0b\x63reate_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12/\n\x0bupdate_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp"T\n\x11\x43reateUserRequest\x12\r\n\x05phone\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12\x10\n\x08username\x18\x03 \x01(\t\x12\x10\n\x08password\x18\x04 \x01(\t"5\n\x17UpdateUserAvatarRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0e\n\x06\x61vatar\x18\x02 \x01(\t"z\n\x11UpdateUserRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x10\n\x08username\x18\x02 \x01(\t\x12\x14\n\x0cold_password\x18\x03 \x01(\t\x12\x14\n\x0cnew_password\x18\x04 \x01(\t\x12\r\n\x05phone\x18\x05 \x01(\t\x12\x0c\n\x04\x63ode\x18\x06 \x01(\t"\x19\n\nUserIdList\x12\x0b\n\x03ids\x18\x01 \x03(\t"-\n\x0cUserInfoList\x12\x1d\n\x05users\x18\x01 \x03(\x0b\x32\x0e.user.UserInfo"9\n\x18UploadUserAvatarResponse\x12\x1d\n\x05token\x18\x01 \x01(\x0b\x32\x0e.util.OssToken2\xe6\x01\n\x0bUserService\x12+\n\x0bGetUserInfo\x12\x0c.user.UserId\x1a\x0e.user.UserInfo\x12/\n\rGetUserDetail\x12\x0c.user.UserId\x1a\x10.user.UserDetail\x12\x34\n\x0cListUserInfo\x12\x10.user.UserIdList\x1a\x12.user.UserInfoList\x12\x43\n\x10UpdateUserAvatar\x12\x1d.user.UpdateUserAvatarRequest\x1a\x10.user.UserDetail2\x81\x02\n\x11UserPublicService\x12+\n\x0bGetUserInfo\x12\x0c.user.UserId\x1a\x0e.user.UserInfo\x12:\n\x0eGetCurrentUser\x12\x16.google.protobuf.Empty\x1a\x10.user.UserDetail\x12\x37\n\nUpdateUser\x12\x17.user.UpdateUserRequest\x1a\x10.user.UserDetail\x12J\n\x10UploadUserAvatar\x12\x16.google.protobuf.Empty\x1a\x1e.user.UploadUserAvatarResponseb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "paperpilot_common.protobuf.user.user_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_USERID"]._serialized_start = 160
    _globals["_USERID"]._serialized_end = 180
    _globals["_USERINFO"]._serialized_start = 182
    _globals["_USERINFO"]._serialized_end = 238
    _globals["_USERDETAIL"]._serialized_start = 241
    _globals["_USERDETAIL"]._serialized_end = 412
    _globals["_CREATEUSERREQUEST"]._serialized_start = 414
    _globals["_CREATEUSERREQUEST"]._serialized_end = 498
    _globals["_UPDATEUSERAVATARREQUEST"]._serialized_start = 500
    _globals["_UPDATEUSERAVATARREQUEST"]._serialized_end = 553
    _globals["_UPDATEUSERREQUEST"]._serialized_start = 555
    _globals["_UPDATEUSERREQUEST"]._serialized_end = 677
    _globals["_USERIDLIST"]._serialized_start = 679
    _globals["_USERIDLIST"]._serialized_end = 704
    _globals["_USERINFOLIST"]._serialized_start = 706
    _globals["_USERINFOLIST"]._serialized_end = 751
    _globals["_UPLOADUSERAVATARRESPONSE"]._serialized_start = 753
    _globals["_UPLOADUSERAVATARRESPONSE"]._serialized_end = 810
    _globals["_USERSERVICE"]._serialized_start = 813
    _globals["_USERSERVICE"]._serialized_end = 1043
    _globals["_USERPUBLICSERVICE"]._serialized_start = 1046
    _globals["_USERPUBLICSERVICE"]._serialized_end = 1303
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import google.protobuf.timestamp_pb2
import paperpilot_common.protobuf.user.user_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class TestResult(google.protobuf.message.Message):
    """测试接口返回"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TIME_FIELD_NUMBER: builtins.int
    USER_FIELD_NUMBER: builtins.int
    @property
    def time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """服务器当前时间"""
    @property
    def user(self) -> paperpilot_common.protobuf.user.user_pb2.UserInfo:
        """（可选）当前登录用户简要信息"""
    def __init__(
        self,
        *,
        time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        user: paperpilot_common.protobuf.user.user_pb2.UserInfo | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["time", b"time", "user", b"user"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["time", b"time", "user", b"user"]) -> None: ...

global___TestResult = TestResult



"""
Django Admin Logs - App Config.
"""
from django.apps import AppConfig
from django.db import models


class DjangoAdminLogsConfig(AppConfig):
    name = "paperpilot_common.utils.admin_logs"
    verbose_name = "Django Admin Logs"

    def ready(self):
        from django.contrib.admin.models import LogEntry

        from .models import NoUserEntryManager

        # Change the model manager to one that doesn't log user
        LogEntry.objects = NoUserEntryManager(LogEntry)

        from django.contrib.admin.templatetags.log import AdminLogNode

        from .render import custom_render

        AdminLogNode.render = custom_render

        from django.contrib.admin.models import LogEntry

        LogEntry.user = None
        LogEntry.user_id = models.IntegerField(default=0, verbose_name="user")



import sys

from loguru import logger as default_logger

_logger = default_logger.bind(title="main")

default_fmt = (
    "<green>{time:HH:mm:ss}</green> <red>|</red> "
    "<level>{level.icon}</level> <red>|</red> "
    # "<cyan>{name}</cyan>:<cyan>{function}</cyan> <red>|</red> "
    "<cyan>[{extra[title]}]</cyan> <red>-</red> <level>{message}</level>"
)


def configure_logging(fmt: str, level: str):
    _logger.remove()
    _logger.add(sys.stdout, format=fmt, level=level)


# configure_logging(settings.LOG_FORMAT, settings.LOG_LEVEL)


def get_logger(title: str | None = None):
    """
    Get logger
    :param title: title of logger (default: None)
    :return: logger
    """
    return _logger.bind(title=title) if title else _logger



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import collections.abc
import google.protobuf.empty_pb2
import grpc
import grpc.aio
import paperpilot_common.protobuf.user.user_pb2
import typing

_T = typing.TypeVar("_T")

class _MaybeAsyncIterator(collections.abc.AsyncIterator[_T], collections.abc.Iterator[_T], metaclass=abc.ABCMeta): ...

class _ServicerContext(grpc.ServicerContext, grpc.aio.ServicerContext):  # type: ignore
    ...

class UserServiceStub:
    """用户服务接口"""

    def __init__(self, channel: typing.Union[grpc.Channel, grpc.aio.Channel]) -> None: ...
    GetUserInfo: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.user_pb2.UserId,
        paperpilot_common.protobuf.user.user_pb2.UserInfo,
    ]
    """获取指定ID用户简要信息"""
    GetUserDetail: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.user_pb2.UserId,
        paperpilot_common.protobuf.user.user_pb2.UserDetail,
    ]
    """获取指定ID用户详细信息"""
    ListUserInfo: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.user_pb2.UserIdList,
        paperpilot_common.protobuf.user.user_pb2.UserInfoList,
    ]
    """批量获取用户简要信息"""
    UpdateUserAvatar: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.user_pb2.UpdateUserAvatarRequest,
        paperpilot_common.protobuf.user.user_pb2.UserDetail,
    ]
    """更新用户头像"""

class UserServiceAsyncStub:
    """用户服务接口"""

    GetUserInfo: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.user_pb2.UserId,
        paperpilot_common.protobuf.user.user_pb2.UserInfo,
    ]
    """获取指定ID用户简要信息"""
    GetUserDetail: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.user_pb2.UserId,
        paperpilot_common.protobuf.user.user_pb2.UserDetail,
    ]
    """获取指定ID用户详细信息"""
    ListUserInfo: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.user_pb2.UserIdList,
        paperpilot_common.protobuf.user.user_pb2.UserInfoList,
    ]
    """批量获取用户简要信息"""
    UpdateUserAvatar: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.user_pb2.UpdateUserAvatarRequest,
        paperpilot_common.protobuf.user.user_pb2.UserDetail,
    ]
    """更新用户头像"""

class UserServiceServicer(metaclass=abc.ABCMeta):
    """用户服务接口"""

    @abc.abstractmethod
    def GetUserInfo(
        self,
        request: paperpilot_common.protobuf.user.user_pb2.UserId,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.user.user_pb2.UserInfo,
        collections.abc.Awaitable[paperpilot_common.protobuf.user.user_pb2.UserInfo],
    ]:
        """获取指定ID用户简要信息"""
    @abc.abstractmethod
    def GetUserDetail(
        self,
        request: paperpilot_common.protobuf.user.user_pb2.UserId,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.user.user_pb2.UserDetail,
        collections.abc.Awaitable[paperpilot_common.protobuf.user.user_pb2.UserDetail],
    ]:
        """获取指定ID用户详细信息"""
    @abc.abstractmethod
    def ListUserInfo(
        self,
        request: paperpilot_common.protobuf.user.user_pb2.UserIdList,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.user.user_pb2.UserInfoList,
        collections.abc.Awaitable[paperpilot_common.protobuf.user.user_pb2.UserInfoList],
    ]:
        """批量获取用户简要信息"""
    @abc.abstractmethod
    def UpdateUserAvatar(
        self,
        request: paperpilot_common.protobuf.user.user_pb2.UpdateUserAvatarRequest,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.user.user_pb2.UserDetail,
        collections.abc.Awaitable[paperpilot_common.protobuf.user.user_pb2.UserDetail],
    ]:
        """更新用户头像"""

def add_UserServiceServicer_to_server(
    servicer: UserServiceServicer, server: typing.Union[grpc.Server, grpc.aio.Server]
) -> None: ...

class UserPublicServiceStub:
    """用户公开接口"""

    def __init__(self, channel: typing.Union[grpc.Channel, grpc.aio.Channel]) -> None: ...
    GetUserInfo: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.user_pb2.UserId,
        paperpilot_common.protobuf.user.user_pb2.UserInfo,
    ]
    """获取指定ID用户简要信息"""
    GetCurrentUser: grpc.UnaryUnaryMultiCallable[
        google.protobuf.empty_pb2.Empty,
        paperpilot_common.protobuf.user.user_pb2.UserDetail,
    ]
    """获取当前用户信息"""
    UpdateUser: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.user_pb2.UpdateUserRequest,
        paperpilot_common.protobuf.user.user_pb2.UserDetail,
    ]
    """更新用户信息"""
    UploadUserAvatar: grpc.UnaryUnaryMultiCallable[
        google.protobuf.empty_pb2.Empty,
        paperpilot_common.protobuf.user.user_pb2.UploadUserAvatarResponse,
    ]
    """上传用户头像"""

class UserPublicServiceAsyncStub:
    """用户公开接口"""

    GetUserInfo: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.user_pb2.UserId,
        paperpilot_common.protobuf.user.user_pb2.UserInfo,
    ]
    """获取指定ID用户简要信息"""
    GetCurrentUser: grpc.aio.UnaryUnaryMultiCallable[
        google.protobuf.empty_pb2.Empty,
        paperpilot_common.protobuf.user.user_pb2.UserDetail,
    ]
    """获取当前用户信息"""
    UpdateUser: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.user.user_pb2.UpdateUserRequest,
        paperpilot_common.protobuf.user.user_pb2.UserDetail,
    ]
    """更新用户信息"""
    UploadUserAvatar: grpc.aio.UnaryUnaryMultiCallable[
        google.protobuf.empty_pb2.Empty,
        paperpilot_common.protobuf.user.user_pb2.UploadUserAvatarResponse,
    ]
    """上传用户头像"""

class UserPublicServiceServicer(metaclass=abc.ABCMeta):
    """用户公开接口"""

    @abc.abstractmethod
    def GetUserInfo(
        self,
        request: paperpilot_common.protobuf.user.user_pb2.UserId,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.user.user_pb2.UserInfo,
        collections.abc.Awaitable[paperpilot_common.protobuf.user.user_pb2.UserInfo],
    ]:
        """获取指定ID用户简要信息"""
    @abc.abstractmethod
    def GetCurrentUser(
        self,
        request: google.protobuf.empty_pb2.Empty,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.user.user_pb2.UserDetail,
        collections.abc.Awaitable[paperpilot_common.protobuf.user.user_pb2.UserDetail],
    ]:
        """获取当前用户信息"""
    @abc.abstractmethod
    def UpdateUser(
        self,
        request: paperpilot_common.protobuf.user.user_pb2.UpdateUserRequest,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.user.user_pb2.UserDetail,
        collections.abc.Awaitable[paperpilot_common.protobuf.user.user_pb2.UserDetail],
    ]:
        """更新用户信息"""
    @abc.abstractmethod
    def UploadUserAvatar(
        self,
        request: google.protobuf.empty_pb2.Empty,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.user.user_pb2.UploadUserAvatarResponse,
        collections.abc.Awaitable[paperpilot_common.protobuf.user.user_pb2.UploadUserAvatarResponse],
    ]:
        """上传用户头像"""

def add_UserPublicServiceServicer_to_server(
    servicer: UserPublicServiceServicer, server: typing.Union[grpc.Server, grpc.aio.Server]
) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class Date(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    YEAR_FIELD_NUMBER: builtins.int
    MONTH_FIELD_NUMBER: builtins.int
    DAY_FIELD_NUMBER: builtins.int
    year: builtins.int
    month: builtins.int
    day: builtins.int
    def __init__(
        self,
        *,
        year: builtins.int = ...,
        month: builtins.int = ...,
        day: builtins.int = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["day", b"day", "month", b"month", "year", b"year"]
    ) -> None: ...

global___Date = Date

@typing_extensions.final
class OssToken(google.protobuf.message.Message):
    """Oss直传Token"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACCESS_KEY_ID_FIELD_NUMBER: builtins.int
    POLICY_FIELD_NUMBER: builtins.int
    SIGNATURE_FIELD_NUMBER: builtins.int
    HOST_FIELD_NUMBER: builtins.int
    CALLBACK_BODY_FIELD_NUMBER: builtins.int
    access_key_id: builtins.str
    """oss access key id"""
    policy: builtins.str
    """上传参数"""
    signature: builtins.str
    """签名"""
    host: builtins.str
    """直传地址"""
    callback_body: builtins.str
    """回调参数"""
    def __init__(
        self,
        *,
        access_key_id: builtins.str = ...,
        policy: builtins.str = ...,
        signature: builtins.str = ...,
        host: builtins.str = ...,
        callback_body: builtins.str = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "access_key_id",
            b"access_key_id",
            "callback_body",
            b"callback_body",
            "host",
            b"host",
            "policy",
            b"policy",
            "signature",
            b"signature",
        ],
    ) -> None: ...

global___OssToken = OssToken



from typing import Tuple
from uuid import UUID


class FakeServicerContext:
    """
    Implementation of basic RpcContext methods that stores data
    for validation in tests
    """

    def __init__(self):
        self.abort_status = None
        self.abort_message = ""
        self._invocation_metadata = tuple()
        self._trailing_metadata = dict()

    def abort(self, status, message):
        """
        gRPC method that is called on RPC exit
        """
        self.abort_status = status
        self.abort_message = message
        raise Exception()  # Just like original context

    def set_trailing_metadata(self, items: Tuple[Tuple[str, str]]):
        """
        gRPC method that is called to set response metadata
        """
        self._trailing_metadata = {d[0]: d[1] for d in items}

    def get_trailing_metadata(self, key: str):
        """
        Helper to retrieve response metadata value by key
        """
        return self._trailing_metadata[key]

    def invocation_metadata(self) -> Tuple[Tuple[str, str]]:
        """
        gRPC method that retrieves request metadata
        """
        return self._invocation_metadata

    def set_invocation_metadata(self, items: Tuple[Tuple[str, str]]):
        """
        Helper to emulate request metadata
        """
        self._invocation_metadata = items

    def authenticate(self, user_id: UUID | str):
        """
        add user_id to metadata
        """
        if isinstance(user_id, UUID):
            user_id = user_id.hex

        from paperpilot_common.middleware.server.auth import UserContext, user_context

        user_context.set(UserContext(user_id))



from typing import Any

from django.conf import settings
from django.core.cache import caches

from paperpilot_common.utils.log import get_logger


class Cache:
    base_key_name: str = settings.SERVER_NAME
    cache_name: str = "default"
    logger_name: str = "cache"
    default_timeout: int | None = None

    def __init__(self):
        self.cache = caches[self.cache_name]
        self.async_cache = caches[f"async_{self.cache_name}"]
        self.logger = get_logger(self.logger_name)

        self.logger.debug(f"cache: '{self.cache_name}' init")

    def key(self, key: str) -> str:
        return f"{self.base_key_name}:{key}"

    def get(self, key: str, default=None) -> Any:
        result = self.cache.get(self.key(key), default)
        self.logger.debug(f"get key: {self.key(key)}, result: {result}")
        return result

    def set(self, key: str, value: Any, timeout: int = default_timeout) -> None:
        self.logger.debug(f"set key: {self.key(key)}, value: {value}, timeout: {timeout}")
        self.cache.set(self.key(key), value, timeout)

    def delete(self, key: str) -> None:
        self.logger.debug(f"delete key: {self.key(key)}")
        self.cache.delete(self.key(key))

    async def get_async(self, key: str, default=None) -> Any:
        result = await self.async_cache.aget(self.key(key), default)
        self.logger.debug(f"async get key: {self.key(key)}, result: {result}")
        return result

    async def set_async(self, key: str, value: Any, timeout: int = default_timeout) -> None:
        self.logger.debug(f"async set key: {self.key(key)}, value: {value}, timeout: {timeout}")
        await self.async_cache.aset(key=self.key(key), value=value, timeout=timeout)

    async def delete_async(self, key: str) -> None:
        self.logger.debug(f"async delete key: {self.key(key)}")
        await self.async_cache.adelete(self.key(key))



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import paperpilot_common.protobuf.common.util_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class UserId(google.protobuf.message.Message):
    """用户 ID"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    id: builtins.str
    """用户 UUID"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["id", b"id"]) -> None: ...

global___UserId = UserId

@typing_extensions.final
class UserInfo(google.protobuf.message.Message):
    """用户简要信息 用于展示"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    USERNAME_FIELD_NUMBER: builtins.int
    AVATAR_FIELD_NUMBER: builtins.int
    id: builtins.str
    """用户 UUID"""
    username: builtins.str
    """用户名"""
    avatar: builtins.str
    """头像"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        username: builtins.str = ...,
        avatar: builtins.str = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["avatar", b"avatar", "id", b"id", "username", b"username"]
    ) -> None: ...

global___UserInfo = UserInfo

@typing_extensions.final
class UserDetail(google.protobuf.message.Message):
    """用户详细信息"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    USERNAME_FIELD_NUMBER: builtins.int
    PHONE_FIELD_NUMBER: builtins.int
    AVATAR_FIELD_NUMBER: builtins.int
    CREATE_TIME_FIELD_NUMBER: builtins.int
    UPDATE_TIME_FIELD_NUMBER: builtins.int
    id: builtins.str
    """用户 UUID"""
    username: builtins.str
    """用户名"""
    phone: builtins.str
    """手机号"""
    avatar: builtins.str
    """头像"""
    @property
    def create_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """创建时间"""
    @property
    def update_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """更新时间"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        username: builtins.str = ...,
        phone: builtins.str = ...,
        avatar: builtins.str = ...,
        create_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        update_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing_extensions.Literal["create_time", b"create_time", "update_time", b"update_time"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "avatar",
            b"avatar",
            "create_time",
            b"create_time",
            "id",
            b"id",
            "phone",
            b"phone",
            "update_time",
            b"update_time",
            "username",
            b"username",
        ],
    ) -> None: ...

global___UserDetail = UserDetail

@typing_extensions.final
class CreateUserRequest(google.protobuf.message.Message):
    """创建用户请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PHONE_FIELD_NUMBER: builtins.int
    CODE_FIELD_NUMBER: builtins.int
    USERNAME_FIELD_NUMBER: builtins.int
    PASSWORD_FIELD_NUMBER: builtins.int
    phone: builtins.str
    """手机号"""
    code: builtins.str
    """验证码"""
    username: builtins.str
    """用户名"""
    password: builtins.str
    """密码"""
    def __init__(
        self,
        *,
        phone: builtins.str = ...,
        code: builtins.str = ...,
        username: builtins.str = ...,
        password: builtins.str = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "code", b"code", "password", b"password", "phone", b"phone", "username", b"username"
        ],
    ) -> None: ...

global___CreateUserRequest = CreateUserRequest

@typing_extensions.final
class UpdateUserAvatarRequest(google.protobuf.message.Message):
    """更新用户头像请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    AVATAR_FIELD_NUMBER: builtins.int
    id: builtins.str
    """用户 UUID"""
    avatar: builtins.str
    """头像"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        avatar: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["avatar", b"avatar", "id", b"id"]) -> None: ...

global___UpdateUserAvatarRequest = UpdateUserAvatarRequest

@typing_extensions.final
class UpdateUserRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    USERNAME_FIELD_NUMBER: builtins.int
    OLD_PASSWORD_FIELD_NUMBER: builtins.int
    NEW_PASSWORD_FIELD_NUMBER: builtins.int
    PHONE_FIELD_NUMBER: builtins.int
    CODE_FIELD_NUMBER: builtins.int
    id: builtins.str
    """用户 UUID"""
    username: builtins.str
    """用户名"""
    old_password: builtins.str
    """旧密码"""
    new_password: builtins.str
    """新密码"""
    phone: builtins.str
    """手机号"""
    code: builtins.str
    """验证码"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        username: builtins.str = ...,
        old_password: builtins.str = ...,
        new_password: builtins.str = ...,
        phone: builtins.str = ...,
        code: builtins.str = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "code",
            b"code",
            "id",
            b"id",
            "new_password",
            b"new_password",
            "old_password",
            b"old_password",
            "phone",
            b"phone",
            "username",
            b"username",
        ],
    ) -> None: ...

global___UpdateUserRequest = UpdateUserRequest

@typing_extensions.final
class UserIdList(google.protobuf.message.Message):
    """用户 ID 列表"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    IDS_FIELD_NUMBER: builtins.int
    @property
    def ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        ids: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["ids", b"ids"]) -> None: ...

global___UserIdList = UserIdList

@typing_extensions.final
class UserInfoList(google.protobuf.message.Message):
    """用户简要信息列表"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    USERS_FIELD_NUMBER: builtins.int
    @property
    def users(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___UserInfo]: ...
    def __init__(
        self,
        *,
        users: collections.abc.Iterable[global___UserInfo] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["users", b"users"]) -> None: ...

global___UserInfoList = UserInfoList

@typing_extensions.final
class UploadUserAvatarResponse(google.protobuf.message.Message):
    """上传头像响应"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TOKEN_FIELD_NUMBER: builtins.int
    @property
    def token(self) -> paperpilot_common.protobuf.common.util_pb2.OssToken: ...
    def __init__(
        self,
        *,
        token: paperpilot_common.protobuf.common.util_pb2.OssToken | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["token", b"token"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["token", b"token"]) -> None: ...

global___UploadUserAvatarResponse = UploadUserAvatarResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class IMAuthResponse(google.protobuf.message.Message):
    """IM认证响应"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    SIG_FIELD_NUMBER: builtins.int
    id: builtins.str
    """user id"""
    sig: builtins.str
    """user sig"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        sig: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["id", b"id", "sig", b"sig"]) -> None: ...

global___IMAuthResponse = IMAuthResponse

@typing_extensions.final
class CreateUserRequest(google.protobuf.message.Message):
    """新建用户请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    USERNAME_FIELD_NUMBER: builtins.int
    AVATAR_FIELD_NUMBER: builtins.int
    id: builtins.str
    """用户id"""
    username: builtins.str
    """用户名"""
    avatar: builtins.str
    """用户头像"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        username: builtins.str = ...,
        avatar: builtins.str = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["avatar", b"avatar", "id", b"id", "username", b"username"]
    ) -> None: ...

global___CreateUserRequest = CreateUserRequest

@typing_extensions.final
class UpdateUserRequest(google.protobuf.message.Message):
    """更新用户请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    USERNAME_FIELD_NUMBER: builtins.int
    AVATAR_FIELD_NUMBER: builtins.int
    id: builtins.str
    """用户id"""
    username: builtins.str
    """用户名"""
    avatar: builtins.str
    """用户头像"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        username: builtins.str = ...,
        avatar: builtins.str = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["avatar", b"avatar", "id", b"id", "username", b"username"]
    ) -> None: ...

global___UpdateUserRequest = UpdateUserRequest

@typing_extensions.final
class CreateWorkGroupRequest(google.protobuf.message.Message):
    """创建Work群组请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    OWNER_FIELD_NUMBER: builtins.int
    id: builtins.str
    """群组id"""
    name: builtins.str
    """群组名称"""
    owner: builtins.str
    """群主id"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        name: builtins.str = ...,
        owner: builtins.str = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["id", b"id", "name", b"name", "owner", b"owner"]
    ) -> None: ...

global___CreateWorkGroupRequest = CreateWorkGroupRequest

@typing_extensions.final
class UpdateWorkGroupRequest(google.protobuf.message.Message):
    """更新Work群组请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    id: builtins.str
    """群组id"""
    name: builtins.str
    """群组名称"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["id", b"id", "name", b"name"]) -> None: ...

global___UpdateWorkGroupRequest = UpdateWorkGroupRequest

@typing_extensions.final
class DeleteWorkGroupRequest(google.protobuf.message.Message):
    """删除Work群组请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    id: builtins.str
    """群组id"""
    def __init__(
        self,
        *,
        id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["id", b"id"]) -> None: ...

global___DeleteWorkGroupRequest = DeleteWorkGroupRequest

@typing_extensions.final
class InviteUserToGroupRequest(google.protobuf.message.Message):
    """邀请用户加入群组请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    GROUP_ID_FIELD_NUMBER: builtins.int
    USER_ID_FIELD_NUMBER: builtins.int
    group_id: builtins.str
    """群组id"""
    user_id: builtins.str
    """用户id"""
    def __init__(
        self,
        *,
        group_id: builtins.str = ...,
        user_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["group_id", b"group_id", "user_id", b"user_id"]
    ) -> None: ...

global___InviteUserToGroupRequest = InviteUserToGroupRequest

@typing_extensions.final
class RemoveUserFromGroupRequest(google.protobuf.message.Message):
    """从群组中移除用户请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    GROUP_ID_FIELD_NUMBER: builtins.int
    USER_ID_FIELD_NUMBER: builtins.int
    group_id: builtins.str
    """群组id"""
    user_id: builtins.str
    """用户id"""
    def __init__(
        self,
        *,
        group_id: builtins.str = ...,
        user_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["group_id", b"group_id", "user_id", b"user_id"]
    ) -> None: ...

global___RemoveUserFromGroupRequest = RemoveUserFromGroupRequest



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import collections.abc
import google.protobuf.empty_pb2
import grpc
import grpc.aio
import paperpilot_common.protobuf.im.im_pb2
import typing

_T = typing.TypeVar("_T")

class _MaybeAsyncIterator(collections.abc.AsyncIterator[_T], collections.abc.Iterator[_T], metaclass=abc.ABCMeta): ...

class _ServicerContext(grpc.ServicerContext, grpc.aio.ServicerContext):  # type: ignore
    ...

class IMServiceStub:
    """即时通讯接口"""

    def __init__(self, channel: typing.Union[grpc.Channel, grpc.aio.Channel]) -> None: ...
    CreateUser: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.im.im_pb2.CreateUserRequest,
        google.protobuf.empty_pb2.Empty,
    ]
    """新建用户"""
    UpdateUser: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.im.im_pb2.UpdateUserRequest,
        google.protobuf.empty_pb2.Empty,
    ]
    """更新用户"""
    CreateWorkGroup: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.im.im_pb2.CreateWorkGroupRequest,
        google.protobuf.empty_pb2.Empty,
    ]
    """创建Work群组"""
    UpdateWorkGroup: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.im.im_pb2.UpdateWorkGroupRequest,
        google.protobuf.empty_pb2.Empty,
    ]
    """更新Work群组"""
    DeleteWorkGroup: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.im.im_pb2.DeleteWorkGroupRequest,
        google.protobuf.empty_pb2.Empty,
    ]
    """删除Work群组"""
    InviteUserToGroup: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.im.im_pb2.InviteUserToGroupRequest,
        google.protobuf.empty_pb2.Empty,
    ]
    """邀请用户加入群组"""
    RemoveUserFromGroup: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.im.im_pb2.RemoveUserFromGroupRequest,
        google.protobuf.empty_pb2.Empty,
    ]
    """从群组中移除用户"""

class IMServiceAsyncStub:
    """即时通讯接口"""

    CreateUser: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.im.im_pb2.CreateUserRequest,
        google.protobuf.empty_pb2.Empty,
    ]
    """新建用户"""
    UpdateUser: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.im.im_pb2.UpdateUserRequest,
        google.protobuf.empty_pb2.Empty,
    ]
    """更新用户"""
    CreateWorkGroup: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.im.im_pb2.CreateWorkGroupRequest,
        google.protobuf.empty_pb2.Empty,
    ]
    """创建Work群组"""
    UpdateWorkGroup: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.im.im_pb2.UpdateWorkGroupRequest,
        google.protobuf.empty_pb2.Empty,
    ]
    """更新Work群组"""
    DeleteWorkGroup: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.im.im_pb2.DeleteWorkGroupRequest,
        google.protobuf.empty_pb2.Empty,
    ]
    """删除Work群组"""
    InviteUserToGroup: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.im.im_pb2.InviteUserToGroupRequest,
        google.protobuf.empty_pb2.Empty,
    ]
    """邀请用户加入群组"""
    RemoveUserFromGroup: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.im.im_pb2.RemoveUserFromGroupRequest,
        google.protobuf.empty_pb2.Empty,
    ]
    """从群组中移除用户"""

class IMServiceServicer(metaclass=abc.ABCMeta):
    """即时通讯接口"""

    @abc.abstractmethod
    def CreateUser(
        self,
        request: paperpilot_common.protobuf.im.im_pb2.CreateUserRequest,
        context: _ServicerContext,
    ) -> typing.Union[google.protobuf.empty_pb2.Empty, collections.abc.Awaitable[google.protobuf.empty_pb2.Empty]]:
        """新建用户"""
    @abc.abstractmethod
    def UpdateUser(
        self,
        request: paperpilot_common.protobuf.im.im_pb2.UpdateUserRequest,
        context: _ServicerContext,
    ) -> typing.Union[google.protobuf.empty_pb2.Empty, collections.abc.Awaitable[google.protobuf.empty_pb2.Empty]]:
        """更新用户"""
    @abc.abstractmethod
    def CreateWorkGroup(
        self,
        request: paperpilot_common.protobuf.im.im_pb2.CreateWorkGroupRequest,
        context: _ServicerContext,
    ) -> typing.Union[google.protobuf.empty_pb2.Empty, collections.abc.Awaitable[google.protobuf.empty_pb2.Empty]]:
        """创建Work群组"""
    @abc.abstractmethod
    def UpdateWorkGroup(
        self,
        request: paperpilot_common.protobuf.im.im_pb2.UpdateWorkGroupRequest,
        context: _ServicerContext,
    ) -> typing.Union[google.protobuf.empty_pb2.Empty, collections.abc.Awaitable[google.protobuf.empty_pb2.Empty]]:
        """更新Work群组"""
    @abc.abstractmethod
    def DeleteWorkGroup(
        self,
        request: paperpilot_common.protobuf.im.im_pb2.DeleteWorkGroupRequest,
        context: _ServicerContext,
    ) -> typing.Union[google.protobuf.empty_pb2.Empty, collections.abc.Awaitable[google.protobuf.empty_pb2.Empty]]:
        """删除Work群组"""
    @abc.abstractmethod
    def InviteUserToGroup(
        self,
        request: paperpilot_common.protobuf.im.im_pb2.InviteUserToGroupRequest,
        context: _ServicerContext,
    ) -> typing.Union[google.protobuf.empty_pb2.Empty, collections.abc.Awaitable[google.protobuf.empty_pb2.Empty]]:
        """邀请用户加入群组"""
    @abc.abstractmethod
    def RemoveUserFromGroup(
        self,
        request: paperpilot_common.protobuf.im.im_pb2.RemoveUserFromGroupRequest,
        context: _ServicerContext,
    ) -> typing.Union[google.protobuf.empty_pb2.Empty, collections.abc.Awaitable[google.protobuf.empty_pb2.Empty]]:
        """从群组中移除用户"""

def add_IMServiceServicer_to_server(
    servicer: IMServiceServicer, server: typing.Union[grpc.Server, grpc.aio.Server]
) -> None: ...

class IMPublicServiceStub:
    """即时通讯公开接口"""

    def __init__(self, channel: typing.Union[grpc.Channel, grpc.aio.Channel]) -> None: ...
    IMAuth: grpc.UnaryUnaryMultiCallable[
        google.protobuf.empty_pb2.Empty,
        paperpilot_common.protobuf.im.im_pb2.IMAuthResponse,
    ]
    """IM认证"""

class IMPublicServiceAsyncStub:
    """即时通讯公开接口"""

    IMAuth: grpc.aio.UnaryUnaryMultiCallable[
        google.protobuf.empty_pb2.Empty,
        paperpilot_common.protobuf.im.im_pb2.IMAuthResponse,
    ]
    """IM认证"""

class IMPublicServiceServicer(metaclass=abc.ABCMeta):
    """即时通讯公开接口"""

    @abc.abstractmethod
    def IMAuth(
        self,
        request: google.protobuf.empty_pb2.Empty,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.im.im_pb2.IMAuthResponse,
        collections.abc.Awaitable[paperpilot_common.protobuf.im.im_pb2.IMAuthResponse],
    ]:
        """IM认证"""

def add_IMPublicServiceServicer_to_server(
    servicer: IMPublicServiceServicer, server: typing.Union[grpc.Server, grpc.aio.Server]
) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import collections.abc
import google.protobuf.empty_pb2
import google.protobuf.wrappers_pb2
import grpc
import grpc.aio
import paperpilot_common.protobuf.project.project_pb2
import typing

_T = typing.TypeVar("_T")

class _MaybeAsyncIterator(collections.abc.AsyncIterator[_T], collections.abc.Iterator[_T], metaclass=abc.ABCMeta): ...

class _ServicerContext(grpc.ServicerContext, grpc.aio.ServicerContext):  # type: ignore
    ...

class ProjectServiceStub:
    """项目接口"""

    def __init__(self, channel: typing.Union[grpc.Channel, grpc.aio.Channel]) -> None: ...
    CheckUserJoinedProject: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.project.project_pb2.CheckUserJoinedProjectRequest,
        google.protobuf.wrappers_pb2.BoolValue,
    ]
    """检查用户是否加入项目"""

class ProjectServiceAsyncStub:
    """项目接口"""

    CheckUserJoinedProject: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.project.project_pb2.CheckUserJoinedProjectRequest,
        google.protobuf.wrappers_pb2.BoolValue,
    ]
    """检查用户是否加入项目"""

class ProjectServiceServicer(metaclass=abc.ABCMeta):
    """项目接口"""

    @abc.abstractmethod
    def CheckUserJoinedProject(
        self,
        request: paperpilot_common.protobuf.project.project_pb2.CheckUserJoinedProjectRequest,
        context: _ServicerContext,
    ) -> typing.Union[
        google.protobuf.wrappers_pb2.BoolValue, collections.abc.Awaitable[google.protobuf.wrappers_pb2.BoolValue]
    ]:
        """检查用户是否加入项目"""

def add_ProjectServiceServicer_to_server(
    servicer: ProjectServiceServicer, server: typing.Union[grpc.Server, grpc.aio.Server]
) -> None: ...

class ProjectPublicServiceStub:
    """项目公开接口"""

    def __init__(self, channel: typing.Union[grpc.Channel, grpc.aio.Channel]) -> None: ...
    ListUserJoinedProjects: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.project.project_pb2.ListProjectRequest,
        paperpilot_common.protobuf.project.project_pb2.ListProjectResponse,
    ]
    """获取用户参与的项目列表"""
    GetProjectInfo: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.project.project_pb2.ProjectId,
        paperpilot_common.protobuf.project.project_pb2.ProjectInfo,
    ]
    """获取项目信息"""
    CreateProject: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.project.project_pb2.CreateProjectRequest,
        paperpilot_common.protobuf.project.project_pb2.ProjectInfo,
    ]
    """创建项目"""
    UpdateProjectInfo: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.project.project_pb2.UpdateProjectRequest,
        paperpilot_common.protobuf.project.project_pb2.ProjectInfo,
    ]
    """修改项目信息"""
    DeleteProject: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.project.project_pb2.ProjectId,
        google.protobuf.empty_pb2.Empty,
    ]
    """删除项目"""
    JoinProject: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.project.project_pb2.ProjectInviteCode,
        paperpilot_common.protobuf.project.project_pb2.ProjectInfo,
    ]
    """加入项目"""
    QuitProject: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.project.project_pb2.ProjectId,
        google.protobuf.empty_pb2.Empty,
    ]
    """退出项目"""

class ProjectPublicServiceAsyncStub:
    """项目公开接口"""

    ListUserJoinedProjects: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.project.project_pb2.ListProjectRequest,
        paperpilot_common.protobuf.project.project_pb2.ListProjectResponse,
    ]
    """获取用户参与的项目列表"""
    GetProjectInfo: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.project.project_pb2.ProjectId,
        paperpilot_common.protobuf.project.project_pb2.ProjectInfo,
    ]
    """获取项目信息"""
    CreateProject: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.project.project_pb2.CreateProjectRequest,
        paperpilot_common.protobuf.project.project_pb2.ProjectInfo,
    ]
    """创建项目"""
    UpdateProjectInfo: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.project.project_pb2.UpdateProjectRequest,
        paperpilot_common.protobuf.project.project_pb2.ProjectInfo,
    ]
    """修改项目信息"""
    DeleteProject: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.project.project_pb2.ProjectId,
        google.protobuf.empty_pb2.Empty,
    ]
    """删除项目"""
    JoinProject: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.project.project_pb2.ProjectInviteCode,
        paperpilot_common.protobuf.project.project_pb2.ProjectInfo,
    ]
    """加入项目"""
    QuitProject: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.project.project_pb2.ProjectId,
        google.protobuf.empty_pb2.Empty,
    ]
    """退出项目"""

class ProjectPublicServiceServicer(metaclass=abc.ABCMeta):
    """项目公开接口"""

    @abc.abstractmethod
    def ListUserJoinedProjects(
        self,
        request: paperpilot_common.protobuf.project.project_pb2.ListProjectRequest,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.project.project_pb2.ListProjectResponse,
        collections.abc.Awaitable[paperpilot_common.protobuf.project.project_pb2.ListProjectResponse],
    ]:
        """获取用户参与的项目列表"""
    @abc.abstractmethod
    def GetProjectInfo(
        self,
        request: paperpilot_common.protobuf.project.project_pb2.ProjectId,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.project.project_pb2.ProjectInfo,
        collections.abc.Awaitable[paperpilot_common.protobuf.project.project_pb2.ProjectInfo],
    ]:
        """获取项目信息"""
    @abc.abstractmethod
    def CreateProject(
        self,
        request: paperpilot_common.protobuf.project.project_pb2.CreateProjectRequest,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.project.project_pb2.ProjectInfo,
        collections.abc.Awaitable[paperpilot_common.protobuf.project.project_pb2.ProjectInfo],
    ]:
        """创建项目"""
    @abc.abstractmethod
    def UpdateProjectInfo(
        self,
        request: paperpilot_common.protobuf.project.project_pb2.UpdateProjectRequest,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.project.project_pb2.ProjectInfo,
        collections.abc.Awaitable[paperpilot_common.protobuf.project.project_pb2.ProjectInfo],
    ]:
        """修改项目信息"""
    @abc.abstractmethod
    def DeleteProject(
        self,
        request: paperpilot_common.protobuf.project.project_pb2.ProjectId,
        context: _ServicerContext,
    ) -> typing.Union[google.protobuf.empty_pb2.Empty, collections.abc.Awaitable[google.protobuf.empty_pb2.Empty]]:
        """删除项目"""
    @abc.abstractmethod
    def JoinProject(
        self,
        request: paperpilot_common.protobuf.project.project_pb2.ProjectInviteCode,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.project.project_pb2.ProjectInfo,
        collections.abc.Awaitable[paperpilot_common.protobuf.project.project_pb2.ProjectInfo],
    ]:
        """加入项目"""
    @abc.abstractmethod
    def QuitProject(
        self,
        request: paperpilot_common.protobuf.project.project_pb2.ProjectId,
        context: _ServicerContext,
    ) -> typing.Union[google.protobuf.empty_pb2.Empty, collections.abc.Awaitable[google.protobuf.empty_pb2.Empty]]:
        """退出项目"""

def add_ProjectPublicServiceServicer_to_server(
    servicer: ProjectPublicServiceServicer, server: typing.Union[grpc.Server, grpc.aio.Server]
) -> None: ...



import abc
import collections.abc
import typing
from typing import Dict, List, Union

import grpc
import grpc.aio

_T = typing.TypeVar("_T")


class _MaybeAsyncIterator(collections.abc.AsyncIterator[_T], collections.abc.Iterator[_T], metaclass=abc.ABCMeta):
    pass


class _ServicerContext(grpc.ServicerContext, grpc.aio.ServicerContext):
    pass


SimpleValue = Union[str, int, bool, float]

JSONValue = Union[None, SimpleValue, List["JSONVal"], Dict[str, "JSONVal"]]  # noqa: F821



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import google.protobuf.timestamp_pb2
import paperpilot_common.protobuf.user.user_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class LoginRequest(google.protobuf.message.Message):
    """登录请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PHONE_FIELD_NUMBER: builtins.int
    PASSWORD_FIELD_NUMBER: builtins.int
    phone: builtins.str
    """手机号"""
    password: builtins.str
    """密码"""
    def __init__(
        self,
        *,
        phone: builtins.str = ...,
        password: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["password", b"password", "phone", b"phone"]) -> None: ...

global___LoginRequest = LoginRequest

@typing_extensions.final
class Token(google.protobuf.message.Message):
    """token"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUE_FIELD_NUMBER: builtins.int
    EXPIRE_TIME_FIELD_NUMBER: builtins.int
    value: builtins.str
    """token值"""
    @property
    def expire_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """过期时间"""
    def __init__(
        self,
        *,
        value: builtins.str = ...,
        expire_time: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["expire_time", b"expire_time"]) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["expire_time", b"expire_time", "value", b"value"]
    ) -> None: ...

global___Token = Token

@typing_extensions.final
class LoginResponse(google.protobuf.message.Message):
    """登录响应"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    USER_FIELD_NUMBER: builtins.int
    ACCESS_FIELD_NUMBER: builtins.int
    REFRESH_FIELD_NUMBER: builtins.int
    @property
    def user(self) -> paperpilot_common.protobuf.user.user_pb2.UserInfo:
        """用户信息"""
    @property
    def access(self) -> global___Token:
        """access token"""
    @property
    def refresh(self) -> global___Token:
        """refresh token"""
    def __init__(
        self,
        *,
        user: paperpilot_common.protobuf.user.user_pb2.UserInfo | None = ...,
        access: global___Token | None = ...,
        refresh: global___Token | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing_extensions.Literal["access", b"access", "refresh", b"refresh", "user", b"user"]
    ) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["access", b"access", "refresh", b"refresh", "user", b"user"]
    ) -> None: ...

global___LoginResponse = LoginResponse

@typing_extensions.final
class RefreshTokenRequest(google.protobuf.message.Message):
    """刷新token请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REFRESH_FIELD_NUMBER: builtins.int
    refresh: builtins.str
    """refresh token"""
    def __init__(
        self,
        *,
        refresh: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["refresh", b"refresh"]) -> None: ...

global___RefreshTokenRequest = RefreshTokenRequest

@typing_extensions.final
class RefreshTokenResponse(google.protobuf.message.Message):
    """刷新token响应"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACCESS_FIELD_NUMBER: builtins.int
    @property
    def access(self) -> global___Token:
        """access token"""
    def __init__(
        self,
        *,
        access: global___Token | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["access", b"access"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["access", b"access"]) -> None: ...

global___RefreshTokenResponse = RefreshTokenResponse

@typing_extensions.final
class SendSmsCodeRequest(google.protobuf.message.Message):
    """发送短信验证码请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PHONE_FIELD_NUMBER: builtins.int
    phone: builtins.str
    """手机号"""
    def __init__(
        self,
        *,
        phone: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["phone", b"phone"]) -> None: ...

global___SendSmsCodeRequest = SendSmsCodeRequest

@typing_extensions.final
class CountPhoneRequest(google.protobuf.message.Message):
    """获取手机号数量请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PHONE_FIELD_NUMBER: builtins.int
    phone: builtins.str
    """手机号"""
    def __init__(
        self,
        *,
        phone: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["phone", b"phone"]) -> None: ...

global___CountPhoneRequest = CountPhoneRequest

@typing_extensions.final
class CountResponse(google.protobuf.message.Message):
    """获取数量响应"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COUNT_FIELD_NUMBER: builtins.int
    count: builtins.int
    """数量"""
    def __init__(
        self,
        *,
        count: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["count", b"count"]) -> None: ...

global___CountResponse = CountResponse

@typing_extensions.final
class CountUsernameRequest(google.protobuf.message.Message):
    """获取用户名数量请求"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    USERNAME_FIELD_NUMBER: builtins.int
    username: builtins.str
    """用户名"""
    def __init__(
        self,
        *,
        username: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["username", b"username"]) -> None: ...

global___CountUsernameRequest = CountUsernameRequest



from pathlib import Path

import watchfiles
from django.conf import settings
from django.utils import autoreload


class WatchfilesReloader(autoreload.BaseReloader):
    def tick(self):
        watcher = watchfiles.watch(*settings.WATCHFILES, debug=False)
        for file_changes in watcher:
            for _change, path in file_changes:
                self.notify_file_changed(Path(path))
            yield


def replaced_get_reloader() -> autoreload.BaseReloader:
    return WatchfilesReloader()


autoreload.get_reloader = replaced_get_reloader



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2

from paperpilot_common.protobuf.user import user_pb2 as paperpilot__common_dot_protobuf_dot_user_dot_user__pb2


class UserServiceStub(object):
    """用户服务接口"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetUserInfo = channel.unary_unary(
            "/user.UserService/GetUserInfo",
            request_serializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserId.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserInfo.FromString,
        )
        self.GetUserDetail = channel.unary_unary(
            "/user.UserService/GetUserDetail",
            request_serializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserId.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserDetail.FromString,
        )
        self.ListUserInfo = channel.unary_unary(
            "/user.UserService/ListUserInfo",
            request_serializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserIdList.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserInfoList.FromString,
        )
        self.UpdateUserAvatar = channel.unary_unary(
            "/user.UserService/UpdateUserAvatar",
            request_serializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UpdateUserAvatarRequest.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserDetail.FromString,
        )


class UserServiceServicer(object):
    """用户服务接口"""

    def GetUserInfo(self, request, context):
        """获取指定ID用户简要信息"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetUserDetail(self, request, context):
        """获取指定ID用户详细信息"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListUserInfo(self, request, context):
        """批量获取用户简要信息"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdateUserAvatar(self, request, context):
        """更新用户头像"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_UserServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "GetUserInfo": grpc.unary_unary_rpc_method_handler(
            servicer.GetUserInfo,
            request_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserId.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserInfo.SerializeToString,
        ),
        "GetUserDetail": grpc.unary_unary_rpc_method_handler(
            servicer.GetUserDetail,
            request_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserId.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserDetail.SerializeToString,
        ),
        "ListUserInfo": grpc.unary_unary_rpc_method_handler(
            servicer.ListUserInfo,
            request_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserIdList.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserInfoList.SerializeToString,
        ),
        "UpdateUserAvatar": grpc.unary_unary_rpc_method_handler(
            servicer.UpdateUserAvatar,
            request_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UpdateUserAvatarRequest.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserDetail.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler("user.UserService", rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class UserService(object):
    """用户服务接口"""

    @staticmethod
    def GetUserInfo(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/user.UserService/GetUserInfo",
            paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserId.SerializeToString,
            paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserInfo.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetUserDetail(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/user.UserService/GetUserDetail",
            paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserId.SerializeToString,
            paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserDetail.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListUserInfo(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/user.UserService/ListUserInfo",
            paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserIdList.SerializeToString,
            paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserInfoList.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UpdateUserAvatar(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/user.UserService/UpdateUserAvatar",
            paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UpdateUserAvatarRequest.SerializeToString,
            paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserDetail.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class UserPublicServiceStub(object):
    """用户公开接口"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetUserInfo = channel.unary_unary(
            "/user.UserPublicService/GetUserInfo",
            request_serializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserId.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserInfo.FromString,
        )
        self.GetCurrentUser = channel.unary_unary(
            "/user.UserPublicService/GetCurrentUser",
            request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserDetail.FromString,
        )
        self.UpdateUser = channel.unary_unary(
            "/user.UserPublicService/UpdateUser",
            request_serializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UpdateUserRequest.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserDetail.FromString,
        )
        self.UploadUserAvatar = channel.unary_unary(
            "/user.UserPublicService/UploadUserAvatar",
            request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            response_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UploadUserAvatarResponse.FromString,
        )


class UserPublicServiceServicer(object):
    """用户公开接口"""

    def GetUserInfo(self, request, context):
        """获取指定ID用户简要信息"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetCurrentUser(self, request, context):
        """获取当前用户信息"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdateUser(self, request, context):
        """更新用户信息"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UploadUserAvatar(self, request, context):
        """上传用户头像"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_UserPublicServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "GetUserInfo": grpc.unary_unary_rpc_method_handler(
            servicer.GetUserInfo,
            request_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserId.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserInfo.SerializeToString,
        ),
        "GetCurrentUser": grpc.unary_unary_rpc_method_handler(
            servicer.GetCurrentUser,
            request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserDetail.SerializeToString,
        ),
        "UpdateUser": grpc.unary_unary_rpc_method_handler(
            servicer.UpdateUser,
            request_deserializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UpdateUserRequest.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserDetail.SerializeToString,
        ),
        "UploadUserAvatar": grpc.unary_unary_rpc_method_handler(
            servicer.UploadUserAvatar,
            request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            response_serializer=paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UploadUserAvatarResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler("user.UserPublicService", rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class UserPublicService(object):
    """用户公开接口"""

    @staticmethod
    def GetUserInfo(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/user.UserPublicService/GetUserInfo",
            paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserId.SerializeToString,
            paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserInfo.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetCurrentUser(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/user.UserPublicService/GetCurrentUser",
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserDetail.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UpdateUser(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/user.UserPublicService/UpdateUser",
            paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UpdateUserRequest.SerializeToString,
            paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UserDetail.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UploadUserAvatar(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/user.UserPublicService/UploadUserAvatar",
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            paperpilot__common_dot_protobuf_dot_user_dot_user__pb2.UploadUserAvatarResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import collections.abc
import google.protobuf.empty_pb2
import grpc
import grpc.aio
import paperpilot_common.protobuf.paper.paper_pb2
import typing

_T = typing.TypeVar("_T")

class _MaybeAsyncIterator(collections.abc.AsyncIterator[_T], collections.abc.Iterator[_T], metaclass=abc.ABCMeta): ...

class _ServicerContext(grpc.ServicerContext, grpc.aio.ServicerContext):  # type: ignore
    ...

class PaperServiceStub:
    def __init__(self, channel: typing.Union[grpc.Channel, grpc.aio.Channel]) -> None: ...
    AddPaper: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
    ]
    """添加论文"""
    UpdatePaper: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
    ]
    """更新论文"""
    UpdateAttachment: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.UpdateAttachmentRequest,
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
    ]
    """更新论文附件"""

class PaperServiceAsyncStub:
    AddPaper: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
    ]
    """添加论文"""
    UpdatePaper: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
    ]
    """更新论文"""
    UpdateAttachment: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.UpdateAttachmentRequest,
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
    ]
    """更新论文附件"""

class PaperServiceServicer(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def AddPaper(
        self,
        request: paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
        collections.abc.Awaitable[paperpilot_common.protobuf.paper.paper_pb2.PaperDetail],
    ]:
        """添加论文"""
    @abc.abstractmethod
    def UpdatePaper(
        self,
        request: paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
        collections.abc.Awaitable[paperpilot_common.protobuf.paper.paper_pb2.PaperDetail],
    ]:
        """更新论文"""
    @abc.abstractmethod
    def UpdateAttachment(
        self,
        request: paperpilot_common.protobuf.paper.paper_pb2.UpdateAttachmentRequest,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
        collections.abc.Awaitable[paperpilot_common.protobuf.paper.paper_pb2.PaperDetail],
    ]:
        """更新论文附件"""

def add_PaperServiceServicer_to_server(
    servicer: PaperServiceServicer, server: typing.Union[grpc.Server, grpc.aio.Server]
) -> None: ...

class PaperPublicServiceStub:
    """论文公开接口"""

    def __init__(self, channel: typing.Union[grpc.Channel, grpc.aio.Channel]) -> None: ...
    ListPaper: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.ListPaperRequest,
        paperpilot_common.protobuf.paper.paper_pb2.ListPaperResponse,
    ]
    """获取论文列表"""
    GetPaper: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.PaperId,
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
    ]
    """获取论文详情"""
    CreatePaper: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.CreatePaperRequest,
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
    ]
    """添加论文"""
    CreatePaperByLink: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.CreatePaperByLinkRequest,
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
    ]
    """从链接创建论文"""
    UpdatePaper: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
    ]
    """更新论文"""
    UploadAttachment: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.UploadAttachmentRequest,
        paperpilot_common.protobuf.paper.paper_pb2.UploadAttachmentResponse,
    ]
    """上传附件"""
    DeletePaper: grpc.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.PaperId,
        google.protobuf.empty_pb2.Empty,
    ]
    """删除论文"""

class PaperPublicServiceAsyncStub:
    """论文公开接口"""

    ListPaper: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.ListPaperRequest,
        paperpilot_common.protobuf.paper.paper_pb2.ListPaperResponse,
    ]
    """获取论文列表"""
    GetPaper: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.PaperId,
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
    ]
    """获取论文详情"""
    CreatePaper: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.CreatePaperRequest,
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
    ]
    """添加论文"""
    CreatePaperByLink: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.CreatePaperByLinkRequest,
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
    ]
    """从链接创建论文"""
    UpdatePaper: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
    ]
    """更新论文"""
    UploadAttachment: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.UploadAttachmentRequest,
        paperpilot_common.protobuf.paper.paper_pb2.UploadAttachmentResponse,
    ]
    """上传附件"""
    DeletePaper: grpc.aio.UnaryUnaryMultiCallable[
        paperpilot_common.protobuf.paper.paper_pb2.PaperId,
        google.protobuf.empty_pb2.Empty,
    ]
    """删除论文"""

class PaperPublicServiceServicer(metaclass=abc.ABCMeta):
    """论文公开接口"""

    @abc.abstractmethod
    def ListPaper(
        self,
        request: paperpilot_common.protobuf.paper.paper_pb2.ListPaperRequest,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.paper.paper_pb2.ListPaperResponse,
        collections.abc.Awaitable[paperpilot_common.protobuf.paper.paper_pb2.ListPaperResponse],
    ]:
        """获取论文列表"""
    @abc.abstractmethod
    def GetPaper(
        self,
        request: paperpilot_common.protobuf.paper.paper_pb2.PaperId,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
        collections.abc.Awaitable[paperpilot_common.protobuf.paper.paper_pb2.PaperDetail],
    ]:
        """获取论文详情"""
    @abc.abstractmethod
    def CreatePaper(
        self,
        request: paperpilot_common.protobuf.paper.paper_pb2.CreatePaperRequest,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
        collections.abc.Awaitable[paperpilot_common.protobuf.paper.paper_pb2.PaperDetail],
    ]:
        """添加论文"""
    @abc.abstractmethod
    def CreatePaperByLink(
        self,
        request: paperpilot_common.protobuf.paper.paper_pb2.CreatePaperByLinkRequest,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
        collections.abc.Awaitable[paperpilot_common.protobuf.paper.paper_pb2.PaperDetail],
    ]:
        """从链接创建论文"""
    @abc.abstractmethod
    def UpdatePaper(
        self,
        request: paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.paper.paper_pb2.PaperDetail,
        collections.abc.Awaitable[paperpilot_common.protobuf.paper.paper_pb2.PaperDetail],
    ]:
        """更新论文"""
    @abc.abstractmethod
    def UploadAttachment(
        self,
        request: paperpilot_common.protobuf.paper.paper_pb2.UploadAttachmentRequest,
        context: _ServicerContext,
    ) -> typing.Union[
        paperpilot_common.protobuf.paper.paper_pb2.UploadAttachmentResponse,
        collections.abc.Awaitable[paperpilot_common.protobuf.paper.paper_pb2.UploadAttachmentResponse],
    ]:
        """上传附件"""
    @abc.abstractmethod
    def DeletePaper(
        self,
        request: paperpilot_common.protobuf.paper.paper_pb2.PaperId,
        context: _ServicerContext,
    ) -> typing.Union[google.protobuf.empty_pb2.Empty, collections.abc.Awaitable[google.protobuf.empty_pb2.Empty]]:
        """删除论文"""

def add_PaperPublicServiceServicer_to_server(
    servicer: PaperPublicServiceServicer, server: typing.Union[grpc.Server, grpc.aio.Server]
) -> None: ...



import hashlib

from paperpilot_common.utils.cache import Cache


class StubCache(Cache):
    logger_name = "grpc.client.cache"
    default_timeout = 5

    def __init__(self, stub_name: str):
        self.stub_name = stub_name
        self.logger_name = f"{self.logger_name}.{stub_name}"
        super().__init__()

    def _generate_cache_key(self, method, args, kwargs):
        # Generate a unique cache key based on method name, args, and kwargs
        key = f"{self.stub_name}:{method}:{self._calculate_hash(args, kwargs)}"
        return key

    def _calculate_hash(self, args, kwargs):
        # Calculate a hash of the args and kwargs to create a shorter cache key
        data = str((args, kwargs)).encode("utf-8")
        return hashlib.sha512(data).hexdigest()

    async def get_result(self, method, args, kwargs):
        cache_key = self._generate_cache_key(method, args, kwargs)
        return await self.get_async(cache_key)

    async def set_result(self, method, args, kwargs, result):
        cache_key = self._generate_cache_key(method, args, kwargs)
        await self.set_async(cache_key, result, timeout=self.default_timeout)

    async def delete_result(self, method, args, kwargs):
        cache_key = self._generate_cache_key(method, args, kwargs)
        await self.delete_async(cache_key)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: paperpilot_common/protobuf/im/im.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n&paperpilot_common/protobuf/im/im.proto\x12\x02im\x1a\x1bgoogle/protobuf/empty.proto")\n\x0eIMAuthResponse\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0b\n\x03sig\x18\x02 \x01(\t"A\n\x11\x43reateUserRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x10\n\x08username\x18\x02 \x01(\t\x12\x0e\n\x06\x61vatar\x18\x03 \x01(\t"A\n\x11UpdateUserRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x10\n\x08username\x18\x02 \x01(\t\x12\x0e\n\x06\x61vatar\x18\x03 \x01(\t"A\n\x16\x43reateWorkGroupRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\r\n\x05owner\x18\x03 \x01(\t"2\n\x16UpdateWorkGroupRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t"$\n\x16\x44\x65leteWorkGroupRequest\x12\n\n\x02id\x18\x01 \x01(\t"=\n\x18InviteUserToGroupRequest\x12\x10\n\x08group_id\x18\x01 \x01(\t\x12\x0f\n\x07user_id\x18\x02 \x01(\t"?\n\x1aRemoveUserFromGroupRequest\x12\x10\n\x08group_id\x18\x01 \x01(\t\x12\x0f\n\x07user_id\x18\x02 \x01(\t2\xf4\x03\n\tIMService\x12;\n\nCreateUser\x12\x15.im.CreateUserRequest\x1a\x16.google.protobuf.Empty\x12;\n\nUpdateUser\x12\x15.im.UpdateUserRequest\x1a\x16.google.protobuf.Empty\x12\x45\n\x0f\x43reateWorkGroup\x12\x1a.im.CreateWorkGroupRequest\x1a\x16.google.protobuf.Empty\x12\x45\n\x0fUpdateWorkGroup\x12\x1a.im.UpdateWorkGroupRequest\x1a\x16.google.protobuf.Empty\x12\x45\n\x0f\x44\x65leteWorkGroup\x12\x1a.im.DeleteWorkGroupRequest\x1a\x16.google.protobuf.Empty\x12I\n\x11InviteUserToGroup\x12\x1c.im.InviteUserToGroupRequest\x1a\x16.google.protobuf.Empty\x12M\n\x13RemoveUserFromGroup\x12\x1e.im.RemoveUserFromGroupRequest\x1a\x16.google.protobuf.Empty2G\n\x0fIMPublicService\x12\x34\n\x06IMAuth\x12\x16.google.protobuf.Empty\x1a\x12.im.IMAuthResponseb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "paperpilot_common.protobuf.im.im_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_IMAUTHRESPONSE"]._serialized_start = 75
    _globals["_IMAUTHRESPONSE"]._serialized_end = 116
    _globals["_CREATEUSERREQUEST"]._serialized_start = 118
    _globals["_CREATEUSERREQUEST"]._serialized_end = 183
    _globals["_UPDATEUSERREQUEST"]._serialized_start = 185
    _globals["_UPDATEUSERREQUEST"]._serialized_end = 250
    _globals["_CREATEWORKGROUPREQUEST"]._serialized_start = 252
    _globals["_CREATEWORKGROUPREQUEST"]._serialized_end = 317
    _globals["_UPDATEWORKGROUPREQUEST"]._serialized_start = 319
    _globals["_UPDATEWORKGROUPREQUEST"]._serialized_end = 369
    _globals["_DELETEWORKGROUPREQUEST"]._serialized_start = 371
    _globals["_DELETEWORKGROUPREQUEST"]._serialized_end = 407
    _globals["_INVITEUSERTOGROUPREQUEST"]._serialized_start = 409
    _globals["_INVITEUSERTOGROUPREQUEST"]._serialized_end = 470
    _globals["_REMOVEUSERFROMGROUPREQUEST"]._serialized_start = 472
    _globals["_REMOVEUSERFROMGROUPREQUEST"]._serialized_end = 535
    _globals["_IMSERVICE"]._serialized_start = 538
    _globals["_IMSERVICE"]._serialized_end = 1038
    _globals["_IMPUBLICSERVICE"]._serialized_start = 1040
    _globals["_IMPUBLICSERVICE"]._serialized_end = 1111
# @@protoc_insertion_point(module_scope)
